import * as ora from 'ora'
import { Git } from '../util/Git'
import { Templates, Template } from '../config/template.config'
import * as inquirer from 'inquirer'
import * as chalk from 'chalk';
/**
 * @date 2019.12.18
 * @author panyanan
 * @information 创建vue2项目
 */
export class CreateVue2 {
    // 模板
    private template: Template;
    // 项目名称
    private projectName: string;
    // 项目目录
    private targetDir: string;
    constructor(template: Template, projectName: string, targetDir: string) {
        this.template = template;
        this.projectName = projectName;
        this.targetDir = targetDir;
    }
    /**
     * @date 2019.12.19
     * @author panyanan
     * @information 创建
     * @param { Object } commander中设置的option 
     */
    public async create(options: any): Promise<void>{
        try {
            let spinner = ora('create new a project');
            spinner.start(`拉取${this.template.name}模板`);
            // 通过仓库地址和输出目录创建git实例
            const pullGit = new Git(this.template.repo, this.targetDir);
            pullGit.pull();
            spinner.succeed('拉取模板成功');
            // 是否推送到gitLab
            let answer = await inquirer.prompt([
                { name: 'push', type: 'confirm', message: '是否将项目推送到gitlab',}
            ]);
            // 如果推送
            if (answer.push) {
                const { repo } = await inquirer.prompt([
                 { name: 'repo', type: 'input', message: '请输入gitlab项目地址:'}
                ]);
                const pushGit = new Git(repo, this.targetDir);
                try {
                    spinner.start(`正在推送到gitlab: ${repo}`)
                    pushGit.push();
                    spinner.succeed('推送成功')
                } catch (error) {
                    spinner.fail(`推送失败: ${error.message}`);
                }
            }
        } catch (error) {
            console.log(`${chalk.red('创建失败')}: ${error.message}`);           
        }
    }
}