/**
 * @date 2019.12.17
 * @author panyanan
 * @information git工具类 提供git拉取,推送等
 */
import * as execa from 'execa'
import * as assert from 'power-assert'
import * as fs from 'fs-extra'
import * as chalk from 'chalk'
import * as spawn from 'cross-spawn'
export class Git {
    private gitURL: string; // 仓库地址
    private dir: string; // 拉取或推送的目录
    constructor(gitURL: string, dir: string) {
        assert.ok(Git.hasGit, '请先安装Git');
        this.gitURL = gitURL;
        this.dir = dir;
    }
    /**
     * @date 2019.12.17
     * @author panyanan
     * @information 拉取
     */
    pull(): void {
        // 检测目录是否存在
        const dirExits = fs.pathExistsSync(this.dir);
        // 目录存在需要删除,才能clone
        if (dirExits) {
            console.log(`${chalk.red('\n delete ')}${this.dir}`);
            fs.removeSync(this.dir);
        }
        execa.sync('git', ['clone', this.gitURL, this.dir]);
    }
    /**
     * @date 2019.12.17
     * @author panyanan
     * @information 推送
     */
    push(): boolean {
        let res = false;
        const options = {
            cwd: this.dir,
        }
        // remove origin
        spawn.sync('git', ['remote', 'remove', 'origin'], options);
        // set origin 为当前url
        spawn.sync('git', ['remote', 'add', 'origin', this.gitURL], options);
        // 推送
        const { status, stderr, stdout } = spawn.sync('git', ['push', '-u', 'origin', '--all'], options);
        // status 为0时命令执行成功, 为1是命令执行失败
        if (status === 0) {
            res = true;
        } else {
            // 抛异常
            throw new Error(stderr.toString())
        }
        return res;
    }
    /**
     * @date 2019.12.17
     * @author panyanan
     * @information 获取git配置中的用户名及邮箱
     */
    static localGitUserInfo():{ name: string, email: string} {
        // 检测git是否安装
        assert.ok(Git.hasGit, '请安装Git');
        // 获取用户名
        const { stdout: name } = execa.sync('git', ['config', 'user.name']);
        // 获取邮箱
        const { stdout: email } = execa.sync('git', ['config', 'user.email']);
        return { name, email };
    }
    /**
     * @date 2019.12.17
     * @author panyanan
     * @information 是否安装了git
     */
    static get hasGit(): boolean {
        let res = false;
        try {
            // 执行git --help 是否抛异常
            execa.sync('git', ['--help']);
            res = true;
        } catch (error) {}
        return res;
    }
}